// -*- C++ -*-
//
// Copyright (C) 1998, 1999, 2000, 2002  Los Alamos National Laboratory,
// Copyright (C) 1998, 1999, 2000, 2002  CodeSourcery, LLC
//
// This file is part of FreePOOMA.
//
// FreePOOMA is free software; you can redistribute it and/or modify it
// under the terms of the Expat license.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Expat
// license for more details.
//
// You should have received a copy of the Expat license along with
// FreePOOMA; see the file LICENSE.
//

/** @file
 * @ingroup Pooma
 * @brief
 * Pooma Extras for the Portable Expression Template Engine
 */

#ifndef POOMA_POOMA_PETE_EXTRAS_H
#define POOMA_POOMA_PETE_EXTRAS_H

template<int D,class T,class E> class Vector;
template<int DR, int DC, class T, class E> class TinyMatrix;
template<int D, class T, class E> class Tensor;

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------

#include "PETE/PETE.h"
#include "Pooma/PoomaOperatorTags.h"

#include <algorithm>
#include <complex>


using std::complex;


//-----------------------------------------------------------------------------
// Complex unary operators
//-----------------------------------------------------------------------------

// The real, imag, abs, arg, and, norm functions need to be defined 
// specially since they don't return complex numbers.

template<class T>
struct UnaryReturn<complex<T>, FnConj >
{
  typedef complex<T> Type_t;
};

template<class T>
struct UnaryReturn<complex<T>, FnReal>
{
  typedef T Type_t;
};

template<class T>
struct UnaryReturn<complex<T>, FnImag>
{
  typedef T Type_t;
};

template<class T>
struct UnaryReturn<complex<T>, FnArg>
{
  typedef T Type_t;
};

template<class T>
struct UnaryReturn<complex<T>, FnNorm>
{
  typedef T Type_t;
};

template<class T>
struct UnaryReturn<T, FnAbs>
{
  typedef T Type_t;
};

template<class T>
struct UnaryReturn<complex<T>, FnAbs>
{
  typedef T Type_t;
};


//-----------------------------------------------------------------------------
// Complex binary operators
//-----------------------------------------------------------------------------

// Define promotions for complex.

// Astoundingly, the binary operators in the standard do not support mixed
// mode arithmetic. 

template<class T>
struct Promote<complex<T>, complex<T> >
{
  typedef complex<T> Type_t;
};

template<class T>
struct Promote<complex<T>, T>
{
  typedef complex<T> Type_t;
};

template<class T>
struct Promote<T, complex<T> >
{
  typedef complex<T> Type_t;
};

// This version is so pow(complex, int) will work correctly.

template<class T>
struct BinaryReturn<complex<T>, int, FnPow>
{
  typedef complex<T> Type_t;
};


//-----------------------------------------------------------------------------
// Special POOMA functions
//-----------------------------------------------------------------------------

template<class T>
struct UnaryReturn<T, FnPow2>
{
  typedef typename BinaryReturn<T, T, OpMultiply>::Type_t Type_t;
};

template<class T>
struct UnaryReturn<T, FnPow3>
{
  typedef typename BinaryReturn<T, T, OpMultiply>::Type_t Type_t;
};

template<class T>
struct UnaryReturn<T, FnPow4>
{
  typedef typename BinaryReturn<T, T, OpMultiply>::Type_t Type_t;
};


#endif // POOMA_POOMA_PETE_EXTRAS_H

// ACL:rcsinfo
// ----------------------------------------------------------------------
// $RCSfile: PETEExtras.h,v $   $Author: richi $
// $Revision: 1.22 $   $Date: 2005/04/08 18:44:26 $
// ----------------------------------------------------------------------
// ACL:rcsinfo
